#!/usr/bin/perl
 
use PDF::API2;
use Text::ParseWords;
use GD::Barcode::Code39;
use GD::Barcode::QRcode;

# Open CSV File With Employee Data
open(FILE,"carbon based liabilities.csv");
@employees = <FILE>;
close(FILE);

# Remove CSV header
shift @employees;

# Load Primary Template File 
$template = PDF::API2->open("ID Badge Template.pdf");

# Create new PDF file
$newpdf_badges = PDF::API2->new();

# Load Fonts -- Forward slashes should work in Windows and Linux.  I guess.
$helvetica_bold = $newpdf_badges->ttfont("fonts/helvetica-bold.ttf");
$helvetica_black = $newpdf_badges->ttfont("fonts/helvetica-black.ttf");
$helvetica = $newpdf_badges->ttfont("fonts/helvetica.ttf");
$helvetica_condensed = $newpdf_badges->ttfont("fonts/helvetica_condensed.ttf");
$eurostile_bold = $newpdf_badges->ttfont("fonts/eurostile-bold.ttf");
$eurostile_demi = $newpdf_badges->ttfont("fonts/eurostile-demi.ttf");

$count = 1;

foreach $employee (@employees){

    chomp $employee;
    
    # Import page from <TEMPLATE>, <SOURCE PAGE>, <DESTINATION PAGE>... And return a Page
    $page = $newpdf_badges->import_page($template, 1, $count);
    $page->mediabox(4*72,7*72);
    $text = $page->text();
    $gfx = $page->gfx();

    # First Name	Last Name	Title	Department	Employee ID	Mobile	email	Photo Species
    ($firstname,$lastname,$jobtitle,$department,$employeeid,$mobilenumber,$email,$photofile,$species) = quotewords(",", 0, $employee);
    print "Processing $count. $firstname $lastname\n";


    # Print name, title, etc.
    $text->font($helvetica_black,18);
    $text->fillcolor('#2D3E50');

    $textwidth = $text->advancewidth(uc("$firstname $lastname"),%text_state);
    $text->translate(144-($textwidth/2),250);
    $text->text(uc("$firstname $lastname"));

    $text->font($helvetica_bold,12);
    $text->fillcolor('#2D3E50');

    $textwidth = $text->advancewidth(uc("$jobtitle"),%text_state);
    $text->translate(144-($textwidth/2),236);
    $text->text(uc("$jobtitle"));

    $text->font($helvetica_condensed,8);
    $text->translate(157,143);
    $text->text("1 $mobilenumber");

    $text->font($helvetica_condensed,7);
    $text->translate(158,110);
    $text->text(lc($email));

    $text->fillcolor('#FFFFFF');
    $text->font($helvetica_condensed,10);
    $textwidth = $text->advancewidth(uc($department),%text_state);    
    $text->translate(144-($textwidth/2),10);
    $text->text(uc($department));


    # Load employee headshot
    $image = $newpdf_badges->image_jpeg("photos/".$photofile); 
    
    # Image, Position X, Position Y, Width, Height
    $gfx->image($image,84,280,120,132);

    # Load Stamp overlay for species
    if($species eq 'HUMAN'){
        $image = $newpdf_badges->image_png("Stamp Human.png"); 
    }
    elsif($species eq 'NON-HUMAN'){
        $image = $newpdf_badges->image_png("Stamp Non-Human.png"); 
    }
    elsif($species eq 'UNKNOWN'){
        $image = $newpdf_badges->image_png("Stamp Unknown.png"); 
    }
    $gfx->image($image,125,270,278/3,188/3);

    # Employee ID Barcode
    barcode();

    # Contact VCARD QRCODE
    qr_vrcode();
    $count++;
}

sub barcode(){
    $barcode = GD::Barcode::Code39->new("*$employeeid*");
    $barcodestring = $barcode->barcode();
    
    $text->font($eurostile_bold,12);
    $text->fillcolor('#2D3E50'); 

    $textwidth = $text->advancewidth("ID: $employeeid",%text_state);
    $text->translate(144-($textwidth/2),179);
    $text->text("ID: $employeeid");
    
    # Prime some initial variables for the barcode
    $offset_x = 29.2;     # X start position
    $offset_y = 192;     # Y start position
    $barthickness = .8; # The thickness of each line
    $barsize=30;        # The height of the barcode
    #####################################
    
    foreach $char (split //, $barcodestring) {
        $gfx->linewidth($barthickness);
    
        $color = $char == 1 ? '#2D3E50' : '#ffffff';
        $gfx->strokecolor($color);
    
        $gfx->move($offset_x,$offset_y);
        $gfx->line($offset_x,$offset_y + $barsize);
        $gfx->stroke;
        $offset_x = $offset_x + $barthickness;
    }    
}

sub qr_vrcode(){

    $string = qq {BEGIN:VCARD
VERSION:2.1
N:$lastname;$firstname;;;
FN:Dr. $firstname $lastname
ORG:Aperture Sciences
ROLE:$jobtitle
TITLE:$jobtitle
TEL;WORK;VOICE:$mobilenumber
ADR;WORK:;;123 Aperture Way;Menominee, MI;;49858;USA
EMAIL;PREF;INTERNET:$email
REV:20140301T221110Z
END:VCARD};

    #$string = "tommy";
    
    $oGdB = GD::Barcode::QRcode->new($string,{
        Ecc => 'M',
        Version=>14,
        ModuleSize => 1
    });
    
    $mat = $oGdB->barcode();
    
    @matrix = split(/\n/,$mat);
    
    $size = 1.3;
    
    $offset_x=25;
    $offset_y=162;
    
    $x=0;
    $y=0;
    
    foreach $row (@matrix){
        foreach $col (split //, $row){
            $color = $col == 1 ? '#2D3E50' : '#ffffff';
            $gfx->fillcolor($color);
            $gfx->rect($offset_x+$x, $offset_y+$y, $size, $size);
            $gfx->fill();            
            
            $x+=$size;
        }
        $x=0;
        $y-=$size;
    }

}


print "\nSaving Employee Badges.pdf...\n";
$newpdf_badges->saveas("Employee Badges.pdf");